/*
 *  Copyright (C) 2006-2007 Daniel Bessler <danielb@kiba-dock.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Daniel Bessler <danielb@kiba-dock.org>
*/

#ifndef __KIBA_PREFS_H__
#define __KIBA_PREFS_H__

#include <gtk/gtk.h>
#include <glib.h>
#include <glib-object.h>

#include <kiba-types.h>

#define KIBA_PREFS_GLOBAL_FILE "global.xml"

G_BEGIN_DECLS

/**
 * KibaPrefs is a settings database.
 **/

typedef struct _KibaPrefs KibaPrefs;
typedef struct _KibaPrefsFile KibaPrefsFile;
typedef struct _PrefMember PrefMember;
typedef struct _KibaTypeData KibaTypeData;

/* setting changed callback type */
typedef void (*KibaPrefCallback) (const char*, gpointer, gpointer);

typedef enum {
	KIBA_INT = 0,
	KIBA_BOOLEAN,
	KIBA_DOUBLE,
	KIBA_STRING,
	KIBA_LIST,
	KIBA_INVALID
}KibaType;

static const struct { KibaType type; const char *pref_type; } kiba_type_data[] = {
	{ KIBA_INT, "int" },
	{ KIBA_BOOLEAN, "bool" },
	{ KIBA_DOUBLE, "double" },
	{ KIBA_STRING, "char" },
	{ KIBA_LIST, "list" },
	{ KIBA_INVALID, "invalid" }
};

/* type used for initialling settings in the database */
struct _PrefMember {
	const char    *id;             /**< setting name/id */
	const char    *default_value;  /**< default value   */
	KibaType       type;           /**< value type      */
};

typedef struct {
	char *id;
	char *type;
	char *value;
}KibaOption;

KibaPrefs* kiba_prefs_get (void);

KibaPrefsFile* kiba_prefs_get_file (const char *path);

const char* kiba_prefs_get_file_path (KibaPrefsFile *file);

gint kiba_prefs_edge_style_from_string (const char *string);

gint kiba_prefs_background_style_from_string (const char *string);

KibaType kiba_prefs_get_type_from_string (const char *string);

KibaPosition kiba_prefs_get_position_from_string (const char *string);

KibaRGBA kiba_prefs_rgba_from_string (const char *string);

gpointer kiba_prefs_get_pointer (KibaPrefsFile *file,
                                 const char *id,
                                 GError **error);

gint kiba_prefs_get_int (KibaPrefsFile *file,
                         const char *id,
                         GError **error);

gint kiba_prefs_get_bool (KibaPrefsFile *file,
                          const char *id,
                          GError **error);

float kiba_prefs_get_float (KibaPrefsFile *file,
                            const char *id,
                            GError **error);

gchar* kiba_prefs_get_string (KibaPrefsFile *file,
                              const char *id,
                              GError **error);

GSList* kiba_prefs_get_list (KibaPrefsFile *file,
                             const char *id,
                             GError **error);

void kiba_prefs_remove_list_item (KibaPrefsFile *file,
                                  const char *list_id,
                                  char *rem_item);

void kiba_prefs_append_to_list (KibaPrefsFile *file,
                                const char *list_name,
                                const char *new_item);

void kiba_prefs_remove_from_list (KibaPrefsFile *file,
                                  const char *list_name,
                                  const char *remove_item);

GSList* kiba_prefs_get_list_from_string (const char *data);

char* kiba_prefs_list_to_string(GList *list);

void kiba_prefs_set_option (KibaPrefsFile *file,
                            const char *id,
                            const char *type,
                            const char *value);

void kiba_prefs_add_callback (KibaPrefsFile *file,
                              const char *id,
                              KibaPrefCallback callback,
                              gpointer data,
                              gboolean launch);

void kiba_prefs_remove_callback_by_data (KibaPrefsFile *file,
                                         gpointer data);

void kiba_prefs_remove_options (KibaPrefsFile *file,
                                PrefMember *options);

void kiba_prefs_add_options (KibaPrefsFile *file,
                             PrefMember *options,
                             gint option_elements);

gboolean kiba_prefs_remove_from_config_file (KibaPrefsFile *file,
                                             const char *id,
                                             const char *type);

gboolean kiba_prefs_write_to_config_file (KibaPrefsFile *file,
                                          const char *id,
                                          const char *type,
                                          const char *value);

char* kiba_prefs_get_option_from_file (KibaPrefsFile *file,
                                       const char *id,
                                       const char *type);

void kiba_prefs_update_config_file(KibaPrefs *prefs,
                                   KibaPrefsFile *file);

KibaPrefsFile* kiba_prefs_add_config_file (const char *configPath);

KibaPrefsFile* kiba_prefs_get_general_file (KibaPrefs *prefs);

G_END_DECLS

#endif
