/*
 *  Copyright (C) 2007 Daniel Bessler <danielb@kiba-dock.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author :	Daniel Bessler <danielb@kiba-dock.org>
 *
*/

/**
 * KibaScreen:
 * checks for opened/closed windows and
 * some other screen propertys and emits signals for them.
 */

#ifndef __KIBA_SCREEN_H__
#define __KIBA_SCREEN_H__

#include <glib.h>
#include <gdk/gdk.h>
#include <glib-object.h>

#include <kiba-window.h>

G_BEGIN_DECLS

typedef struct _KibaScreen      KibaScreen;
typedef struct _KibaScreenClass KibaScreenClass;

#define KIBA_TYPE_SCREEN           (kiba_screen_get_type ())
#define KIBA_SCREEN(obj)           (G_TYPE_CHECK_INSTANCE_CAST ((obj), KIBA_TYPE_SCREEN, KibaScreen))
#define KIBA_SCREEN_CLASS(obj)     (G_TYPE_CHECK_CLASS_CAST ((obj), KIBA_SCREEN,  KibaScreenClass))
#define KIBA_IS_SCREEN(obj)        (G_TYPE_CHECK_INSTANCE_TYPE ((obj), KIBA_TYPE_SCREEN))
#define KIBA_IS_SCREEN_CLASS(obj)  (G_TYPE_CHECK_CLASS_TYPE ((obj), KIBA_TYPE_SCREEN))
#define KIBA_SCREEN_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), KIBA_TYPE_SCREEN, KibaScreenClass))

#define KIBA_SCREEN_DEFAULT        kiba_screen_get()
#define KIBA_GDK_SCREEN            KIBA_SCREEN_DEFAULT->gdk_screen

#ifdef __cplusplus
extern "C" {
#endif

struct _KibaScreen {
	GObject       parent_instance;
	
	GdkScreen     *gdk_screen; /**< The GdkScreen associated to the KibaScreen */
	GList         *windows;    /**< List with all KibaWindow's managed by screen */
	GdkRectangle   geometry;   /**< Screen geometry */
	gint           monitor;    /**< Screen monitor or 0 */
};

struct _KibaScreenClass {
	GObjectClass parent_class;
	
	/* signals */
	void (* window_opened) (KibaScreen *screen,
				KibaWindow *window);
	void (* window_closed) (KibaScreen *screen,
				KibaWindow *window);
	void (* withdrawn_toggled) (KibaScreen *screen,
				    KibaWindow *window);
	void (* active_window_changed) (KibaScreen *screen,
				        KibaWindow *active_window);
	void (* viewport_changed) (KibaScreen *screen,
				   GdkPoint   *new_active);
	void (* workspace_changed) (KibaScreen *screen,
				    gint new_active);
	void (* screen_changed) (KibaScreen *screen,
				 GdkScreen  *new_screen);
};

KibaScreen* kiba_screen_get ();

void kiba_screen_set_monitor (KibaScreen *screen,
			      gint        monitor);

KibaWindow* kiba_screen_get_window (KibaScreen *screen,
				    GdkWindow  *gdk_window);

void kiba_screen_destroy (KibaScreen *screen);

void kiba_screen_for_each_window (KibaScreen *screen,
                                  GFunc       for_each_func,
                                  gpointer    data);

GList* kiba_screen_get_stack (KibaScreen *screen);

G_END_DECLS

#endif

