/*
 *  Copyright (C) 2007 Daniel Bessler <danielb@kiba-dock.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Daniel Bessler <danielb@kiba-dock.org>
*/

/**
 * KibaTitle displays text above KibaObject's.
 **/

#ifndef __KIBA_TITLE_H__
#define __KIBA_TITLE_H__

#include <gdk/gdk.h>
#include <cairo.h>

#include <kiba-types.h>
#include <kiba-object.h>

G_BEGIN_DECLS

#define KIBA_TYPE_TITLE           (kiba_title_get_type ())
#define KIBA_TITLE(obj)           (G_TYPE_CHECK_INSTANCE_CAST ((obj), KIBA_TYPE_TITLE, KibaTitle))
#define KIBA_TITLE_CLASS(obj)     (G_TYPE_CHECK_CLASS_CAST ((obj), KIBA_TITLE,  KibaTitleClass))
#define KIBA_IS_TITLE(obj)        (G_TYPE_CHECK_INSTANCE_TYPE ((obj), KIBA_TYPE_TITLE))
#define KIBA_IS_TITLE_CLASS(obj)  (G_TYPE_CHECK_CLASS_TYPE ((obj), KIBA_TYPE_TITLE))
#define KIBA_TITLE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), KIBA_TYPE_TITLE, KibaTitleClass))

typedef struct _KibaTitle KibaTitle;
typedef struct _KibaTitleClass KibaTitleClass;

/**
 * KibaTitleStyle:
 *
 * Title style struct.
 */
typedef struct {
	union {
		KibaBackgroundStyle   style;  /**< The background style */
		char                 *file;   /**< The pattern file path */
	}background;
	
	KibaCornerShape border_style;	/**< Border style */
	float border_width;	/**< Border width */
	float bg_affection;	/**< Affection for linear style */
	float bg_step_size;	/**< Step size for radial/linear style */
	KibaRGBA rgba;		/**< Text color */
	KibaRGBA highlightColor;
	KibaRGBA shadow_rgba;	/**< Background color */
	KibaRGBA border_rgba;	/**< Border color */
	KibaRGBA *gradient;
	gint numColors;
}KibaTitleStyle;

/**
 * KibaTitleFlag:
 *
 * Title flags enum.
 */
typedef enum {
	KIBA_TITLE_SCALING = 0,
	KIBA_TITLE_DRAWABLE,
	KIBA_TITLE_HIGHLIGHTING
}KibaTitleFlag;

KibaTitle* kiba_title_new ();

gboolean kiba_title_is_shown (KibaTitle *title);

void kiba_title_set_style (KibaTitle *title,
                           KibaTitleStyle style);

void kiba_title_set_flag (KibaTitle *title,
                          KibaTitleFlag flag);

void kiba_title_unset_flag (KibaTitle *title,
                            KibaTitleFlag flag);

gboolean kiba_title_get_flag (KibaTitle *title,
                              KibaTitleFlag flag);

void kiba_title_set_font (KibaTitle *title,
                          char *font);

KibaTitleStyle* kiba_title_get_style (KibaTitle *title);

void kiba_title_set_hide_delay (KibaTitle *title,
                                gint hide_delay);

void kiba_title_add_object (KibaTitle *title,
                                        KibaObject *object);

void kiba_title_show_object (KibaTitle *title,
                             KibaObject *object);

void kiba_title_hide_object (KibaTitle *title,
                             KibaObject *object);

gboolean kiba_title_object_is_shown (KibaTitle *title,
                                     KibaObject *object);

void kiba_title_set_object_text (KibaTitle *title,
                                 KibaObject *object,
                                 const char *text);

char* kiba_title_get_object_text (KibaTitle *title,
                                  KibaObject *object);

void kiba_title_remove_object (KibaTitle *title,
                               KibaObject *object);

G_END_DECLS

#endif
