/*
 *  Copyright (C) 2007 Daniel Bessler <danielb@kiba-dock.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Daniel Bessler <danielb@kiba-dock.org>
 *
*/

#ifndef __KIBA_TYPES_H__
#define __KIBA_TYPES_H__

#include <gtk/gtk.h>

G_BEGIN_DECLS

typedef struct _KibaLayoutData KibaLayoutData;
typedef struct _KibaStrut KibaStrut;
typedef struct _KibaRGBA KibaRGBA;
typedef struct _KibaRGB KibaRGB;
typedef struct _KibaAllocation KibaAllocation;

struct _KibaAllocation {
	float x;
	float y;
	float width;
	float height;
};

/**
 * Color type with alpha member.
 */
struct _KibaRGBA {
	float r;
	float g;
	float b;
	float a;
};

/**
 * Color type without alpha member.
 */
struct _KibaRGB {
	float r;
	float g;
	float b;
};

/**
 * Type for struts (_NET_WM_STRUT).
 */
struct _KibaStrut {
	gint left;
	gint right;
	gint top;
	gint bottom;
};

/**
 * Type for the Shape of corners.
 */
typedef enum {
	KIBA_ROUNDED = 0,
	KIBA_MITERED,
	KIBA_SQUARED,
	KIBA_NOTCHED
}KibaCornerShape;

/**
 * Background Style type.
 */
typedef enum {
	KIBA_GRADIENT = 0,  /**< gradient pattern */
	KIBA_RADIAL,        /**< radial pattern */
	KIBA_LINEAR,        /**< linear pattern */
	KIBA_NONE
}KibaBackgroundStyle;

/**
 * Layout type.
 */
struct _KibaLayoutData {
	const char      *name;         /**< The position name */
	GtkOrientation  orientation;   /**< The position orientation */
	gint            halignment;    /**< The horizontal alignment (left=0,center=1,right=2) */
	gint            valignment;    /**< The vertical alignment (top=0,center=1,bottom=2) */
	GdkPoint        anchor;        /**< The position anchor */
};

/**
 * Position type.
 */
typedef enum {
	KIBA_POSITION_LEFT_EDGE,
	KIBA_POSITION_RIGHT_EDGE,
	KIBA_POSITION_TOP_EDGE,
	KIBA_POSITION_BOTTOM_EDGE
}KibaPosition;

/**
 * Gets the layout data with a KibaPosition (&kiba_layout_data[position])
 */
static const KibaLayoutData kiba_layout_data[] = {
	{ "left_edge",    GTK_ORIENTATION_VERTICAL,   0, 1, { 0, 2 } },
	{ "right_edge",   GTK_ORIENTATION_VERTICAL,   2, 1, { 4, 2 } },
	{ "top_edge",     GTK_ORIENTATION_HORIZONTAL, 1, 0, { 2, 0 } },
	{ "bottom_edge",  GTK_ORIENTATION_HORIZONTAL, 1, 2, { 2, 4 } }
};

G_END_DECLS

#endif
