/*
 *  Copyright (C) 2007 Daniel Bessler <danielb@kiba-dock.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Daniel Bessler <danielb@kiba-dock.org>
*/

#include <math.h>

#include "kiba-dock.h"
#include "kiba-separator.h"
#include "dock-private.h"

static void kiba_dock_default_shape_separator_draw (KibaDock *dock,
						    cairo_t  *cr,
						    gpointer  data) {
	const KibaLayoutData *d;
	gint border_width;
	GList *l;
	
	d = &kiba_layout_data[dock->d.position];
	border_width = gtk_container_get_border_width(GTK_CONTAINER(dock));
	
	cairo_set_operator (cr, CAIRO_OPERATOR_ATOP);
	
	cairo_set_source_rgba (cr,
		dock->style.border_rgba.r,
		dock->style.border_rgba.g,
		dock->style.border_rgba.b,
		dock->style.border_rgba.a);
// 	cairo_set_tolerance (cr, 1.0);
	cairo_set_line_width (cr, border_width/2);
	
	for (l = GTK_BOX(dock)->children; l != NULL; l = l->next) {
		GtkBoxChild *child = l->data;
		GtkWidget *separator = child->widget;
		float fX, fY;
		
		if (!KIBA_IS_SEPARATOR(separator))
			continue;
		
		fX = KIBA_OBJECT(separator)->alloc.x;
		fY = KIBA_OBJECT(separator)->alloc.y;
		
		cairo_save(cr);
			if (d->orientation == GTK_ORIENTATION_HORIZONTAL) {
// 				cairo_set_line_width (cr, separator->allocation.width/2);
				cairo_move_to (cr, fX + separator->allocation.width/2, fY);
				cairo_rel_line_to (cr, 0, dock->d.height);
			}
			else {
// 				cairo_set_line_width (cr, separator->allocation.height/2);
				cairo_move_to (cr, fX, fY + separator->allocation.height/2);
				cairo_rel_line_to (cr, dock->d.height, 0);
			}
			cairo_stroke (cr);
		cairo_restore (cr);
	}
}

static void kiba_dock_default_shape_draw (KibaDock *dock,
					  cairo_t  *cr,
					  gpointer  data) {
	cairo_pattern_t  *pattern;
	gint x0, y0, x1, y1;
	gint border_width;
	gint radius;
	const KibaLayoutData *d;
	
	d = &kiba_layout_data[dock->d.position];
	border_width = gtk_container_get_border_width (GTK_CONTAINER(dock));
	radius = dock->shape.corner_radius;
	
	x0 = border_width/2;
	y0 = border_width/2;
	x1 = dock->surface_data.width  - border_width/2;
	y1 = dock->surface_data.height - border_width/2;
	
	/* move to Top-Left */
	cairo_move_to (cr, x0, y0 + radius);
 
	/* Top-Left Corner */
	if (d->anchor.y == 4 || d->anchor.x == 4) {
		switch (dock->shape.topleft) {
			case KIBA_NOTCHED:
				cairo_arc_negative (cr, x0, y0, radius, M_PI / 2, 2 * M_PI);
				break;
			case KIBA_MITERED:
				cairo_line_to (cr, x0 + radius, y0);
				break;
			case KIBA_ROUNDED:
				cairo_arc (cr, x0 + radius, y0 + radius, radius, M_PI, 3 * M_PI / 2);
				break;
			default: /* squared style */
				cairo_line_to (cr, x0, y0);
				cairo_line_to (cr, x0 + radius, y0);
				break;
		}
	}
	else if (d->anchor.y == 0) {
		if (y0 + radius > y1 - radius)
			cairo_move_to (cr, x0, y1 - radius);
 
		cairo_line_to (cr, x0, y0 - border_width);
		cairo_line_to (cr, x0 + 2*radius, y0 - border_width);
	}
	else if (d->anchor.x == 0) {
		cairo_move_to (cr, x0 - border_width, y0);
		if (x0 + radius < x1 - radius)
			cairo_line_to (cr, x0 + radius, y0);
		else
			cairo_line_to (cr, x1 - radius, y0);
	}
	
	if ((d->anchor.x != 4 && d->anchor.x != 0) || (x0 + radius < x1 - radius))
		cairo_rel_line_to (cr, x1 - 2*radius - border_width/2, 0);
	
	/* Top-Right Corner */
	if (d->anchor.y == 4 || d->anchor.x == 0) {
		switch (dock->shape.topright) {
			case KIBA_NOTCHED:
				cairo_arc_negative (cr, x1, y0, radius, M_PI, M_PI/2);
				break;
			case KIBA_MITERED:
				cairo_line_to (cr, x1 - radius, y0);
				cairo_line_to (cr, x1, y0 + radius);
				break;
			case KIBA_ROUNDED:
				cairo_arc (cr, x1 - radius, y0 + radius, radius, 3 * M_PI / 2, 2 * M_PI);
				break;
			default: /* squared style */
				cairo_line_to (cr, x1, y0);
				cairo_line_to (cr, x1, y0 + radius);
				break;
		}
	}
	else if (d->anchor.x == 4) {
		cairo_line_to (cr, x1 + border_width, y0);
		cairo_line_to (cr, x1 + border_width, y0 + 2*radius);
	}
	else if (d->anchor.y == 0 && (y0 + radius < y1 - radius))
		cairo_line_to (cr, x1, y0 + radius);
	
	if ((d->anchor.y != 4 && d->anchor.y != 0) || (y0 + radius < y1 - radius))
		cairo_rel_line_to (cr, 0, y1 - 2*radius - border_width/2);
	
	/* Bottom-Right Corner */
	if (d->anchor.y == 0 || d->anchor.x == 0) {
		switch (dock->shape.bottomright) {
			case KIBA_NOTCHED:
				cairo_arc_negative (cr, x1, y1, radius, 3 * M_PI / 2, M_PI);
				break;
			case KIBA_MITERED:
				cairo_line_to (cr, x1, y1 - radius);
				cairo_line_to (cr, x1 - radius, y1);
				break;
			case KIBA_ROUNDED:
				cairo_arc (cr, x1 - radius, y1 - radius, radius, 0, M_PI / 2);
				break;
			default: /* squared style */
				cairo_line_to (cr, x1, y1);
				cairo_line_to (cr, x1 - radius, y1);
				break;
		}
	}
	else if (d->anchor.y == 4) {
		cairo_line_to (cr, x1, y1 + border_width);
		cairo_line_to (cr, x1 - 2*radius, y1 + border_width);
	}
	else if (d->anchor.x == 4 && (x0 + radius < x1 - radius))
		cairo_line_to (cr, x1 - radius, y1);
	
	if ((d->anchor.x != 4 && d->anchor.x != 0) || (x0 + radius < x1 - radius))
		cairo_rel_line_to (cr, -x1 + 2*radius + border_width/2, 0);
	
	/* Bottom-Left Corner */
	if (d->anchor.y == 0 || d->anchor.x == 4) {
		switch (dock->shape.bottomleft) {
			case KIBA_NOTCHED:
				cairo_arc_negative (cr, x0, y1, radius, 2 * M_PI, 3 * M_PI / 2);
				break;
			case KIBA_MITERED:
				cairo_line_to (cr, x0 + radius, y1);
				cairo_line_to (cr, x0, y1 - radius);
				break;
			case KIBA_ROUNDED:
				cairo_arc (cr, x0 + radius, y1 - radius, radius, M_PI / 2, M_PI);
				break;
			default:
				cairo_line_to (cr, x0, y1);
				cairo_line_to (cr, x0, y1 - radius);
				break;
		}
	}
	else if (d->anchor.x == 0) {
		cairo_line_to (cr, x0 - border_width, y1);
		cairo_line_to (cr, x0 - border_width, y1 - 2*radius);
	}
	else if (d->anchor.y == 4 && (y0 + radius < y1 - radius))
		cairo_line_to (cr, x0, y1 - radius);
	
	if ((d->anchor.y != 4 && d->anchor.y != 0) || (y0 + radius < y1 - radius))
		cairo_rel_line_to (cr, 0, -y1 + 2*radius + border_width/2);
	
	cairo_close_path (cr);
	
	pattern = kiba_dock_get_pattern (dock);
		if (d->anchor.y == 0)
			cairo_rotate (cr, M_PI);
		else if (d->anchor.x == 0)
			cairo_rotate (cr, M_PI/2);
		else if (d->anchor.x == 4)
			cairo_rotate (cr, 3*M_PI/2);
		cairo_set_source (cr, pattern);
		cairo_fill_preserve (cr);
	cairo_pattern_destroy (pattern);
	
	/* draw border */
	cairo_set_source_rgba (cr,
		dock->style.border_rgba.r, dock->style.border_rgba.g,
		dock->style.border_rgba.b, dock->style.border_rgba.a);
	cairo_set_line_width (cr, border_width);
	cairo_set_line_join (cr, CAIRO_LINE_JOIN_ROUND);
	cairo_stroke (cr);
}

void kiba_dock_default_shape_activate (KibaDock *dock) {
	gint border_width;
	GList *l;
	
	border_width = gtk_container_get_border_width(GTK_CONTAINER(dock));
	
	dock->shape.draw_func = kiba_dock_default_shape_draw;
	dock->shape.separator_draw_func = kiba_dock_default_shape_separator_draw;
	dock->shape.clip_func = NULL;
	dock->shape.draw_data = NULL;
	dock->shape.triangle_ontop = TRUE;
	
	//TODO border_width may change
	for (l = GTK_BOX(dock)->children; l != NULL; l = l->next) {
		GtkBoxChild *child = l->data;
		if (!KIBA_IS_SEPARATOR(child->widget))
			continue;
		kiba_separator_set_size(KIBA_SEPARATOR(child->widget), border_width);
	}
}
