/*
 *  Copyright (C) 2007 Daniel Bessler <danielb@kiba-dock.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Daniel Bessler <danielb@kiba-dock.org>
*/

#include "../config.h"

#include "kiba.h"
#include "kiba-object.h"
#include "kiba-separator.h"
#include "kiba-types.h"
#include "kiba-prefs.h"
#include "kiba-screen.h"
#include "kiba-types.h"

#include "prefs-callbacks.h"

static PrefMember kiba_general_options[] = {
	{ "monitor_number", "1", KIBA_INT }, /**< TODO should be a dock setting */
	/* KibaTitle properties */
	{ "title_enable", "TRUE", KIBA_BOOLEAN },
	{ "title_highlighting", "FALSE", KIBA_BOOLEAN },
	{ "title_highlighting_color", "#000000AA", KIBA_STRING },
	{ "title_border_style", "rounded", KIBA_STRING },
	{ "title_text_rgba", "#FFFFFFFF", KIBA_STRING },
	{ "title_shadow_rgba", "#FFFFFF88", KIBA_STRING },
	{ "title_background", "gradient", KIBA_STRING },
	{ "title_gradient", "[#082D7688]", KIBA_LIST },
	{ "title_disable_on_tasks", "FALSE", KIBA_BOOLEAN },
	{ "title_scale_enable", "FALSE", KIBA_BOOLEAN },
	{ "title_border_rgba", "#145BC488", KIBA_STRING },
	{ "title_border_width", "1.0", KIBA_DOUBLE },
	{ "title_hide_delay", "800.0", KIBA_DOUBLE },
	{ "title_font", "Sans Regular 12", KIBA_STRING },
	/* KibaTooltip properties */
	{ "tooltip_font", "Sans 12", KIBA_STRING },
	{ "tooltip_border_width", "5", KIBA_INT },
	{ "tooltip_gradient", "[#082D7688]", KIBA_LIST },
	{ "tooltip_border_rgba", "#000000FF", KIBA_STRING },
	{ "tooltip_text_rgba", "#000000FF", KIBA_STRING },
	{ "tooltip_background", "gradient", KIBA_STRING },
	{ "tooltip_border_style", "rounded", KIBA_STRING },
	/* Popup properties */
	{ "popup_border_color", "#145BC4FF", KIBA_STRING },
	{ "popup_corner_radius", "5.0", KIBA_DOUBLE },
	{ "popup_border_width", "1", KIBA_INT },
	{ "popup_draw", "TRUE", KIBA_BOOLEAN },
	{ "popup_background", "gradient", KIBA_STRING },
	{ "popup_gradient", "[#082D7688]", KIBA_LIST },
	{ "popup_topleft", "rounded", KIBA_STRING },
	{ "popup_topright", "rounded", KIBA_STRING },
	{ "popup_bottomleft", "rounded", KIBA_STRING },
	{ "popup_bottomright", "rounded", KIBA_STRING }
};

static PrefMember kiba_options[] = {
	/* KibaWin properties */
	{ "keep_window_below_others", "FALSE", KIBA_BOOLEAN },
	{ "keep_window_above_others", "FALSE", KIBA_BOOLEAN },
	/* KibaDock properties */
	{ "separator_list", "[]", KIBA_LIST },
	{ "dock_groups", "[]", KIBA_LIST },
	{ "dock_set_strut", "TRUE", KIBA_BOOLEAN },
	{ "dock_shape", "podest", KIBA_STRING },
	{ "edge_position", "bottom_edge", KIBA_STRING },
	{ "dock_max_size", "0", KIBA_INT },
	{ "dock_position_stepsize", "10.0", KIBA_DOUBLE },
	{ "dock_background_alpha", "0.8", KIBA_DOUBLE },
	{ "dock_border_rgba", "#145BC4FF", KIBA_STRING },
	{ "dock_border_width", "1.0", KIBA_DOUBLE },
	{ "dock_cap_size", "5", KIBA_INT },
	{ "dock_draw", "TRUE", KIBA_BOOLEAN },
	{ "dock_curves", "TRUE", KIBA_BOOLEAN },
	{ "dock_background", "gradient", KIBA_STRING },
	{ "dock_curves_color", "#00000033", KIBA_STRING },
	{ "dock_gradient", "[#082D7688]", KIBA_LIST },
	{ "dock_corner_topleft", "rounded", KIBA_STRING },
	{ "dock_corner_topright", "rounded", KIBA_STRING },
	{ "dock_corner_bottomleft", "rounded", KIBA_STRING },
	{ "dock_corner_bottomright", "rounded", KIBA_STRING },
	{ "dock_margin", "40", KIBA_INT },
	{ "dock_edge_radius", "26.0", KIBA_DOUBLE },
	{ "dock_edge_offset", "0", KIBA_INT },
	{ "auto_hide_enable", "FALSE", KIBA_BOOLEAN },
	{ "auto_hide_delay", "5", KIBA_INT },
	{ "auto_hide_speed", "5", KIBA_INT },
	{ "auto_hide_area", "100", KIBA_INT },
	{ "object_space", "15", KIBA_INT },
	{ "object_margin", "10", KIBA_INT },
	{ "dock_object_size", "38", KIBA_INT },
	/* KibaObject properties */
	{ "prelight_enable", "TRUE", KIBA_BOOLEAN },
	{ "prelight_rgba", "#FFFFFF66", KIBA_STRING },
	{ "reflection_enable", "TRUE", KIBA_BOOLEAN },
	{ "reflection_alpha", "0.6", KIBA_DOUBLE },
	{ "reflection_scale_factor", "0.75", KIBA_DOUBLE },
	{ "shadow_enable", "FALSE", KIBA_BOOLEAN },
	{ "object_shadow_color", "#00000088", KIBA_STRING },
	{ "object_shadow_offset", "4", KIBA_INT },
	{ "shadow_affection", "45", KIBA_INT },
	/* KibaTriangle properties */
	{ "notification_triangle_enable", "TRUE", KIBA_BOOLEAN },
	{ "triangle_icon", DATADIR "/kiba-dock/icons/broken-glass.svg", KIBA_STRING },
	/* KibaProgress properties */
	{ "progress_enable", "TRUE", KIBA_BOOLEAN },
	{ "progress_rgba", "#00000066", KIBA_STRING },
	{ "progress_bg_rgba", "#00000066", KIBA_STRING },
	/* Continuous progress properties */
	{ "cprogress_enable", "TRUE", KIBA_BOOLEAN },
	{ "cprogress_style", "default", KIBA_STRING },
	{ "cprogress_size", "26", KIBA_INT },
	{ "cprogress_step", "0.05", KIBA_DOUBLE },
	/* KibaSeparator properties */
	{ "separator_draw", "FALSE", KIBA_BOOLEAN },
	{ "separator_size", "6", KIBA_INT }
};

static gchar* kiba_win_prob_data[] = {
	"keep_window_below_others",     /**< KIBA_WIN_BELOW **/
	"keep_window_above_others"     /**< KIBA_WIN_ABOVE **/
};

/*** Callbacks ***/

static void kiba_win_flag_cb (const char *option_id,
			      gpointer    value,
			      gpointer    data) {
	KibaWin *win = data;
	gint i;
	
	for (i = 0; i < G_N_ELEMENTS(kiba_win_prob_data); i++) {
		if (strcmp (kiba_win_prob_data[i], option_id) != 0)
			continue;
		
		if ((gboolean) GPOINTER_TO_INT(value) == TRUE)
			kiba_win_set_flag (win, i);
		else
			kiba_win_unset_flag (win, i);
	}
}

static void kiba_dock_position_cb (const char *option_id,
			          gpointer    value,
			          gpointer    data) {
	KibaDock *dock = data;
	
	kiba_dock_set_position (dock,
		kiba_prefs_get_position_from_string((char*) value));
}

static void dock_corner_cb (const char *option_id,
			    gpointer    value,
			    gpointer    data) {
	KibaDock *dock = data;
	KibaDockShape shape;
	
	shape = dock->shape;
	
	if (strcmp (option_id, "dock_edge_radius") == 0)
		shape.corner_radius = (float)g_ascii_strtod((char*) value, NULL);
	else {
		KibaCornerShape corner;
		corner =  kiba_prefs_edge_style_from_string ((char*) value);
		
		if (strcmp (option_id, "dock_corner_topleft") == 0)
			shape.topleft = corner;
		else if (strcmp (option_id, "dock_corner_topright") == 0)
			shape.topright = corner;
		else if (strcmp (option_id, "dock_corner_bottomright") == 0)
			shape.bottomright = corner;
		else if (strcmp (option_id, "dock_corner_bottomleft") == 0)
			shape.bottomleft = corner;
	}
	
	kiba_dock_set_shape (dock, shape);
}

static void dock_layout_cb (const char *option_id,
			    gpointer    value,
			    gpointer    data) {
	KibaDock *dock = data;
	KibaDockLayout d;
	
	d = dock->d;
	if (strcmp (option_id, "dock_cap_size") == 0)
		d.cap_size = GPOINTER_TO_INT (value);
	else if (strcmp (option_id, "object_space") == 0)
		d.object_space = GPOINTER_TO_INT (value);
	else if (strcmp (option_id, "object_margin") == 0)
		d.object_margin = GPOINTER_TO_INT (value);
	else if (strcmp (option_id, "dock_margin") == 0)
		d.height = GPOINTER_TO_INT (value);
	else if (strcmp (option_id, "dock_max_size") == 0) {
		d.max_size = GPOINTER_TO_INT (value);
		kiba_dock_set_max_size (dock, d.max_size);
		return;
	}
	
	kiba_dock_set_layout (dock, d);
}

static void dock_object_size_cb (const char *option_id,
			         gpointer    value,
			         gpointer    data) {
	KibaDock *dock = data;
	kiba_dock_set_object_size (dock, GPOINTER_TO_INT(value));
}

static void dock_offset_cb (const char *option_id,
			    gpointer    value,
			    gpointer    data) {
	KibaDock *dock = data;
	
	dock->d.offset = GPOINTER_TO_INT(value);
	gtk_widget_queue_resize (GTK_WIDGET(dock));
}

static void dock_border_width_cb (const char *option_id,
				  gpointer    value,
				  gpointer    data) {
	KibaDock *dock = data;
	
	gtk_container_set_border_width (GTK_CONTAINER(dock),
		(gint) (float)g_ascii_strtod((char*) value, NULL));
}

void kiba_prefs_string_list_to_rgba_array (GSList *list,
                                           KibaRGBA *rgba) {
	GSList *l;
	gint index = 0;
	
	for (l = list; l != NULL; l = l->next) {
		rgba[index] = kiba_prefs_rgba_from_string((char*) l->data);
		index++;
	}
}

static void dock_style_cb (const char *option_id,
			   gpointer    value,
			   gpointer    data) {
	KibaDock *dock = data;
	KibaDockStyle *style;
	
	style = &dock->style;
	
	if (strcmp (option_id, "dock_background") == 0) {
		if (g_str_has_suffix ((char*) value, ".png") || g_str_has_suffix ((char*) value, ".svg"))
			style->background.file = (char*) value;
		else
			style->background.style = kiba_prefs_background_style_from_string ((char*) value);
	}
	else if (strcmp (option_id, "dock_background_alpha") == 0)
		style->background_alpha = (float)g_ascii_strtod((char*) value, NULL);
	else if (strcmp (option_id, "reflection_alpha") == 0)
		style->reflection_alpha = (float)g_ascii_strtod((char*) value, NULL);
	else if (strcmp (option_id, "reflection_scale_factor") == 0)
		style->reflection_scale_factor = (float)g_ascii_strtod((char*) value, NULL);
	else if (strcmp (option_id, "prelight_rgba") == 0)
		style->prelight_rgba = kiba_prefs_rgba_from_string ((char*) value);
	else if (strcmp (option_id, "dock_gradient") == 0) {
		GSList *list = value;
		gint length = g_slist_length(list);
		KibaRGBA *gradient = g_new0(KibaRGBA, length);
		kiba_prefs_string_list_to_rgba_array(list, gradient);
		if (style->gradient)
			g_free (style->gradient);
		style->gradient = gradient;
		style->numColors = length;
	}
	else if (strcmp (option_id, "dock_border_rgba") == 0)
		style->border_rgba = kiba_prefs_rgba_from_string ((char*) value);
	else if (strcmp (option_id, "dock_curves_color") == 0)
		style->curve_color = kiba_prefs_rgba_from_string ((char*) value);
	
	kiba_dock_set_style (dock, *style);
}

static void tooltip_settings_cb (const char *option_id,
				  gpointer    value,
				  gpointer    data) {
	Kiba *kiba = kiba_get();
	KibaTooltip *tooltip;
	
	tooltip = kiba_get_tooltip(kiba);
	
	if (strcmp (option_id, "tooltip_gradient") == 0) {
		GSList *list = value;
		gint length = g_slist_length(list);
		KibaRGBA *gradient = g_new0(KibaRGBA, length);
		kiba_prefs_string_list_to_rgba_array(list, gradient);
		kiba_tooltip_set_gradient(tooltip, gradient, length);
	}
	else if (strcmp (option_id, "tooltip_font") == 0) {
		kiba_tooltip_set_font(tooltip, (char*) value);
	}
	else if (strcmp (option_id, "tooltip_border_width") == 0) {
		kiba_tooltip_set_border_width(tooltip, GPOINTER_TO_INT(value));
	}
	else if (strcmp (option_id, "tooltip_border_rgba") == 0) {
		tooltip->borderColor = kiba_prefs_rgba_from_string ((char*) value);
		gtk_widget_queue_draw(GTK_WIDGET(tooltip));
	}
	else if (strcmp (option_id, "tooltip_text_rgba") == 0) {
		tooltip->textColor = kiba_prefs_rgba_from_string ((char*) value);
		gtk_widget_queue_draw(GTK_WIDGET(tooltip));
	}
	else if (strcmp (option_id, "tooltip_background") == 0) {
		if (g_str_has_suffix ((char*) value, ".png") || g_str_has_suffix ((char*) value, ".svg"))
			tooltip->background.file = (char*) value;
		else
			tooltip->background.style = kiba_prefs_background_style_from_string ((char*) value);
		gtk_widget_queue_draw(GTK_WIDGET(tooltip));
	}
	else if (strcmp (option_id, "tooltip_border_style") == 0) {
		tooltip->cornerStyle = kiba_prefs_edge_style_from_string ((char*) value);
		gtk_widget_queue_draw(GTK_WIDGET(tooltip));
	}
}

static void separator_size_cb (const char *option_id,
			       gpointer    value,
			       gpointer    data) {
	KibaSeparator *separator = data;
	kiba_fixed_object_set_size (KIBA_FIXED_OBJECT(separator),GPOINTER_TO_INT (value));
	gtk_widget_queue_resize (GTK_WIDGET(separator));
}

static void progress_style_cb (const char *option_id,
			       gpointer    value,
			       gpointer    data) {
	KibaProgress *progress = data;
	KibaProgressStyle style;
	
	style = kiba_progress_get_style(progress);
	
	if (strcmp (option_id, "progress_rgba") == 0)
		style.rgba = kiba_prefs_rgba_from_string ((char*) value);
	else if (strcmp (option_id, "progress_bg_rgba") == 0)
		style.bg_rgba = kiba_prefs_rgba_from_string ((char*) value);
	
	kiba_progress_set_style (progress, style);
}

static void title_style_cb (const char *option_id,
			    gpointer    value,
			    gpointer    data) {
	KibaTitle *title = data;
	KibaTitleStyle *style;
	
	style = kiba_title_get_style(title);
	
	if (strcmp (option_id, "title_background") == 0) {
		if (g_str_has_suffix ((char*) value, ".png") || g_str_has_suffix ((char*) value, ".svg"))
			style->background.file = (char*) value;
		else
			style->background.style = kiba_prefs_background_style_from_string ((char*) value);
	}
	else if (strcmp (option_id, "title_border_style") == 0)
		style->border_style = kiba_prefs_edge_style_from_string ((char*) value);
	else if (strcmp (option_id, "title_border_width") == 0)
		style->border_width = (float)g_ascii_strtod((char*) value, NULL);
	else if (strcmp (option_id, "title_gradient") == 0) {
		GSList *list = value;
		gint length = g_slist_length(list);
		KibaRGBA *gradient = g_new0(KibaRGBA, length);
		kiba_prefs_string_list_to_rgba_array(list, gradient);
		if (style->gradient)
			g_free(style->gradient);
		style->gradient = gradient;
		style->numColors = length;
	}
	else if (strcmp (option_id, "title_border_rgba") == 0)
		style->border_rgba = kiba_prefs_rgba_from_string ((char*) value);
	
	kiba_title_set_style (title, *style);
}

static void title_prop_cb (const char *option_id,
			   gpointer    value,
			   gpointer    data) {
	KibaTitle *title = data;
	KibaTitleStyle *style;
	
	style = kiba_title_get_style(title);
	
	if (strcmp (option_id, "title_text_rgba") == 0)
		style->rgba = kiba_prefs_rgba_from_string ((char*) value);
	else if (strcmp (option_id, "title_highlighting_color") == 0)
		style->highlightColor = kiba_prefs_rgba_from_string ((char*) value);
	else if (strcmp (option_id, "title_shadow_rgba") == 0)
		style->shadow_rgba = kiba_prefs_rgba_from_string ((char*) value);
	else if (strcmp (option_id, "title_hide_delay") == 0)
		kiba_title_set_hide_delay(title, (int) (float)g_ascii_strtod((char*) value, NULL));
}

static void title_flag_cb (const char *option_id,
			   gpointer    value,
			   gpointer    data) {
	KibaTitle *title = data;
	gint flag = 0;
	
	if (strcmp (option_id, "title_scale_enable") == 0)
		flag = KIBA_TITLE_SCALING;
	else if (strcmp (option_id, "title_enable") == 0)
		flag = KIBA_TITLE_DRAWABLE;
	else if (strcmp (option_id, "title_highlighting") == 0)
		flag = KIBA_TITLE_HIGHLIGHTING;
	
	if ((gboolean)(GPOINTER_TO_INT (value)) == TRUE)
		kiba_title_set_flag (title, flag);
	else
		kiba_title_unset_flag (title, flag);
}

static void triangle_flag_cb (const char *option_id,
			      gpointer    value,
			      gpointer    data) {
	KibaTriangle *triangle = data;
	gint flag;
	
	if (strcmp (option_id, "notification_triangle_enable") == 0)
		flag = KIBA_TRIANGLE_DRAWABLE;
	
	if ((gboolean)(GPOINTER_TO_INT (value)) == TRUE)
		kiba_triangle_set_flag (triangle, flag);
	else
		kiba_triangle_unset_flag (triangle, flag);
}

static void triangle_icon_cb (const char *option_id,
			      gpointer    value,
			      gpointer    data) {
	KibaTriangle *triangle = data;
	const char *icon = value;
	
	kiba_triangle_set_icon (triangle, icon);
}

typedef enum {
	KIBA_DOCK_SHAPE_STYLE_DEFAULT = 0,
	KIBA_DOCK_SHAPE_STYLE_PODEST,
	KIBA_DOCK_SHAPE_STYLE_NULL
}KibaDockShapeStyle;

static KibaDockShapeStyle kiba_dock_get_shape_style_from_string (const char *style_string) {
	static const char *kiba_dock_style_data[] = {
		"default", "podest", "custom"
	};
	
	gint i;
	
	for (i = 0; i < G_N_ELEMENTS(kiba_dock_style_data); i++) {
		if (strcmp (style_string, kiba_dock_style_data[i]) == 0) {
			return i;
		}
	}
	
	return KIBA_DOCK_SHAPE_STYLE_NULL;
}

static void dock_shape_cb (const char *option_id,
			   gpointer    value,
			   gpointer    data) {
	KibaDock *dock = data;
	KibaDockShapeStyle style;
	
	style = kiba_dock_get_shape_style_from_string ((char*) value);
	
	switch (style) {
		case KIBA_DOCK_SHAPE_STYLE_DEFAULT:
			kiba_dock_default_shape_activate (dock);
			break;
		case KIBA_DOCK_SHAPE_STYLE_PODEST:
			kiba_dock_podest_shape_activate (dock);
			break;
	}
	
	kiba_dock_update_surface (dock);
}

static void dock_flags_cb (const char *option_id,
			   gpointer    value,
			   gpointer    data) {
	KibaDock *dock = data;
	gint flag = 0;
	
	if (strcmp (option_id, "auto_hide_enable") == 0)
		flag = KIBA_DOCK_AUTOHIDE;
	else if (strcmp (option_id, "separator_draw") == 0)
		flag = KIBA_DOCK_DRAW_SEPARATOR;
	else if (strcmp (option_id, "dock_draw") == 0)
		flag = KIBA_DOCK_DRAWABLE;
	else if (strcmp (option_id, "dock_set_strut") == 0)
		flag = KIBA_DOCK_SET_STRUT;
	else if (strcmp (option_id, "dock_curves") == 0)
		flag = KIBA_DOCK_DRAW_CURVES;
	else if (strcmp (option_id, "shadow_enable") == 0)
		flag = KIBA_DOCK_DRAW_SHADOW;
	else if (strcmp (option_id, "prelight_enable") == 0)
		flag = KIBA_DOCK_DRAW_PRELIGHT;
	else if (strcmp (option_id, "reflection_enable") == 0)
		flag = KIBA_DOCK_DRAW_REFLECTIONS;
	else if (strcmp (option_id, "progress_enable") == 0)
		flag = KIBA_DOCK_DRAW_PROGRESS;
	
	if ((gboolean)(GPOINTER_TO_INT (value)) == TRUE)
		kiba_dock_set_flag (dock, flag);
	else
		kiba_dock_unset_flag (dock, flag);
}

static void title_font_cb (const char *option_id,
			    gpointer    value,
			    gpointer    data) {
	KibaTitle *title = data;
	
	kiba_title_set_font (title, (char*) value);
}

static void object_props_cb (const char *option_id,
			     gpointer    value,
			     gpointer    data) {
	KibaObject *object = data;
	KibaDrawableObject *drawable_object = KIBA_DRAWABLE_OBJECT(object);
	
	if (strcmp (option_id, "object_shadow_color") == 0) {
		drawable_object->shadow_color = kiba_prefs_rgba_from_string ((char*) value);
		kiba_drawable_object_queue_draw (drawable_object);
	}
	else if (strcmp (option_id, "object_shadow_offset") == 0) {
		drawable_object->shadow_offset = (GPOINTER_TO_INT (value));
		kiba_drawable_object_queue_draw (drawable_object);
	}
	else if (strcmp (option_id, "shadow_affection") == 0) {
		drawable_object->shadow_affection = (GPOINTER_TO_INT (value));
		kiba_drawable_object_queue_draw (drawable_object);
	}
}

static void cprogress_cb (const char *option_id,
			 gpointer    value,
			 gpointer    data) {
	KibaDock *dock = data;
	
	if (strcmp (option_id, "cprogress_enable") == 0) {
		dock->continuousProgress.enable = (gboolean)GPOINTER_TO_INT(value);
	}
	else if (strcmp (option_id, "cprogress_style") == 0) {
		if (strcmp((char*) value, "dark") == 0)
			kiba_dock_set_continuous_progress_icon(dock, DATADIR "/kiba-dock/icons/gear-dark.svg");
		else
			kiba_dock_set_continuous_progress_icon(dock, DATADIR "/kiba-dock/icons/gear.svg");
	}
	else if (strcmp (option_id, "cprogress_size") == 0) {
		kiba_dock_set_continuous_progress_size(dock, GPOINTER_TO_INT (value));
	}
	else if (strcmp (option_id, "cprogress_step") == 0) {
		dock->continuousProgress.step = (float)g_ascii_strtod((char*) (value), NULL);
	}
	
	gtk_widget_queue_draw(GTK_WIDGET(dock));
}

static void autohide_cb (const char *option_id,
			 gpointer    value,
			 gpointer    data) {
	KibaDock *dock = data;
	
	if (strcmp (option_id, "auto_hide_delay") == 0) {
		dock->autohide.delay = GPOINTER_TO_INT (value);
	}
	else if (strcmp (option_id, "auto_hide_speed") == 0) {
		dock->autohide.speed = GPOINTER_TO_INT (value);
	}
	else if (strcmp (option_id, "auto_hide_area") == 0) {
		dock->autohide.area = GPOINTER_TO_INT (value);
	}
}

static void screen_set_monitor_cb (const char *option_id,
				   gpointer    value,
				   gpointer    data) {
	kiba_screen_set_monitor (KIBA_SCREEN_DEFAULT, GPOINTER_TO_INT (value));
}

/*** Callback adding ***/

static void kiba_remove_object_callbacks (GtkContainer *container,
					  GtkWidget    *widget,
					  gpointer data) {
	KibaPrefsFile *configFile = data;
	kiba_prefs_remove_callback_by_data (configFile, widget);
}

static void kiba_add_object_callbacks (GtkContainer *container,
				       GtkWidget    *widget,
				       gpointer data) {
	KibaPrefsFile *configFile = data;
	if (KIBA_IS_SEPARATOR(widget))
		kiba_prefs_add_callback (configFile, "separator_size", separator_size_cb, widget, TRUE);
	else {
		if (KIBA_IS_DRAWABLE_OBJECT(widget)) {
			kiba_prefs_add_callback (configFile, "object_shadow_color", object_props_cb, KIBA_OBJECT(widget), TRUE);
			kiba_prefs_add_callback (configFile, "object_shadow_offset", object_props_cb, KIBA_OBJECT(widget), TRUE);
			kiba_prefs_add_callback (configFile, "shadow_affection", object_props_cb, KIBA_OBJECT(widget), TRUE);
		}
	}
}

void kiba_prefs_callbacks_add_dock (KibaWin  *window,
			           KibaDock *dock,
			           KibaPrefsFile *file) {
	GList *l;
	
	/* initiall window properties */
	kiba_prefs_add_callback (file, "keep_window_below_others", kiba_win_flag_cb, window, TRUE);
	kiba_prefs_add_callback (file, "keep_window_above_others", kiba_win_flag_cb, window, TRUE);
	
	/* initiall dock properties */
	kiba_prefs_add_callback (file, "edge_position", kiba_dock_position_cb, dock, TRUE);
	kiba_prefs_add_callback (file, "dock_corner_topleft", dock_corner_cb, dock, TRUE);
	kiba_prefs_add_callback (file, "dock_corner_topright", dock_corner_cb, dock, TRUE);
	kiba_prefs_add_callback (file, "dock_corner_bottomleft", dock_corner_cb, dock, TRUE);
	kiba_prefs_add_callback (file, "dock_corner_bottomright", dock_corner_cb, dock, TRUE);
	kiba_prefs_add_callback (file, "dock_edge_radius", dock_corner_cb, dock, TRUE);
	kiba_prefs_add_callback (file, "dock_margin", dock_layout_cb, dock, TRUE);
	kiba_prefs_add_callback (file, "dock_cap_size", dock_layout_cb, dock, TRUE);
	kiba_prefs_add_callback (file, "dock_max_size", dock_layout_cb, dock, TRUE);
	kiba_prefs_add_callback (file, "object_space", dock_layout_cb, dock, TRUE);
	kiba_prefs_add_callback (file, "object_margin", dock_layout_cb, dock, TRUE);
	kiba_prefs_add_callback (file, "dock_background_alpha", dock_style_cb, dock, TRUE);
	kiba_prefs_add_callback (file, "dock_border_rgba", dock_style_cb, dock, TRUE);
	kiba_prefs_add_callback (file, "dock_background", dock_style_cb, dock, TRUE);
	kiba_prefs_add_callback (file, "dock_curves_color", dock_style_cb, dock, TRUE);
	kiba_prefs_add_callback (file, "dock_gradient", dock_style_cb, dock, TRUE);
	kiba_prefs_add_callback (file, "prelight_rgba", dock_style_cb, dock, TRUE);
	kiba_prefs_add_callback (file, "reflection_alpha", dock_style_cb, dock, TRUE);
	kiba_prefs_add_callback (file, "reflection_scale_factor", dock_style_cb, dock, TRUE);
	kiba_prefs_add_callback (file, "dock_border_width", dock_border_width_cb, dock, TRUE);
	kiba_prefs_add_callback (file, "dock_edge_offset", dock_offset_cb, dock, TRUE);
	kiba_prefs_add_callback (file, "auto_hide_enable", dock_flags_cb, dock, TRUE);
	kiba_prefs_add_callback (file, "separator_draw", dock_flags_cb, dock, TRUE);
	kiba_prefs_add_callback (file, "prelight_enable", dock_flags_cb, dock, TRUE);
	kiba_prefs_add_callback (file, "reflection_enable", dock_flags_cb, dock, TRUE);
	kiba_prefs_add_callback (file, "dock_draw", dock_flags_cb, dock, TRUE);
	kiba_prefs_add_callback (file, "progress_enable", dock_flags_cb, dock, TRUE);
	kiba_prefs_add_callback (file, "dock_curves", dock_flags_cb, dock, TRUE);
	kiba_prefs_add_callback (file, "shadow_enable", dock_flags_cb, dock, TRUE);
	kiba_prefs_add_callback (file, "dock_shape", dock_shape_cb, dock, TRUE);
	kiba_prefs_add_callback (file, "dock_set_strut", dock_flags_cb, dock, TRUE);
	kiba_prefs_add_callback (file, "dock_object_size", dock_object_size_cb, dock, TRUE);
	kiba_prefs_add_callback (file, "auto_hide_speed", autohide_cb, dock, TRUE);
	kiba_prefs_add_callback (file, "auto_hide_area", autohide_cb, dock, TRUE);
	kiba_prefs_add_callback (file, "auto_hide_delay", autohide_cb, dock, TRUE);
	kiba_prefs_add_callback (file, "cprogress_enable", cprogress_cb, dock, TRUE);
	kiba_prefs_add_callback (file, "cprogress_style", cprogress_cb, dock, TRUE);
	kiba_prefs_add_callback (file, "cprogress_size", cprogress_cb, dock, TRUE);
	kiba_prefs_add_callback (file, "cprogress_step", cprogress_cb, dock, TRUE);
	
	kiba_prefs_add_callback (file, "notification_triangle_enable", triangle_flag_cb, window->triangle, TRUE);
	kiba_prefs_add_callback (file, "triangle_icon", triangle_icon_cb, window->triangle, TRUE);
	
	kiba_prefs_add_callback (file, "progress_rgba", progress_style_cb, window->progress, TRUE);
	kiba_prefs_add_callback (file, "progress_bg_rgba", progress_style_cb, window->progress, TRUE);
	
	g_signal_connect (G_OBJECT(dock), "add",
		G_CALLBACK(kiba_add_object_callbacks), file);
	g_signal_connect (G_OBJECT(dock), "remove",
		G_CALLBACK(kiba_remove_object_callbacks), file);
}

void prefs_callbacks_add_dock_options (KibaPrefsFile *configFile) {
	kiba_prefs_add_options (configFile, kiba_options, G_N_ELEMENTS(kiba_options));
}

void prefs_callbacks_add_general_options (KibaPrefsFile *generalFile) {
	Kiba *kiba = kiba_get();
	KibaTitle *title = kiba_get_title(kiba);
	
	kiba_prefs_add_options (generalFile, kiba_general_options, G_N_ELEMENTS(kiba_general_options));
	
	kiba_prefs_add_callback (generalFile, "monitor_number", screen_set_monitor_cb, NULL, TRUE);
	
	kiba_prefs_add_callback (generalFile, "tooltip_gradient",
		tooltip_settings_cb, kiba, TRUE);
	kiba_prefs_add_callback (generalFile, "tooltip_font",
		tooltip_settings_cb, kiba, TRUE);
	kiba_prefs_add_callback (generalFile, "tooltip_border_width",
		tooltip_settings_cb, kiba, TRUE);
	kiba_prefs_add_callback (generalFile, "tooltip_border_rgba",
		tooltip_settings_cb, kiba, TRUE);
	kiba_prefs_add_callback (generalFile, "tooltip_text_rgba",
		tooltip_settings_cb, kiba, TRUE);
	kiba_prefs_add_callback (generalFile, "tooltip_background",
		tooltip_settings_cb, kiba, TRUE);
	kiba_prefs_add_callback (generalFile, "tooltip_border_style",
		tooltip_settings_cb, kiba, TRUE);
	
	kiba_prefs_add_callback (generalFile, "title_enable", title_flag_cb, title, TRUE);
	kiba_prefs_add_callback (generalFile, "title_highlighting", title_flag_cb, title, TRUE);
	kiba_prefs_add_callback (generalFile, "title_font", title_font_cb, title, TRUE);
	kiba_prefs_add_callback (generalFile, "title_scale_enable", title_flag_cb, title, TRUE);
	kiba_prefs_add_callback (generalFile, "title_hide_delay", title_prop_cb, title, TRUE);
	kiba_prefs_add_callback (generalFile, "title_text_rgba", title_prop_cb, title, TRUE);
	kiba_prefs_add_callback (generalFile, "title_highlighting_color", title_prop_cb, title, TRUE);
	kiba_prefs_add_callback (generalFile, "title_shadow_rgba", title_prop_cb, title, TRUE);
	kiba_prefs_add_callback (generalFile, "title_background", title_style_cb, title, TRUE);
	kiba_prefs_add_callback (generalFile, "title_gradient", title_style_cb, title, TRUE);
	kiba_prefs_add_callback (generalFile, "title_border_rgba", title_style_cb, title, TRUE);
	kiba_prefs_add_callback (generalFile, "title_border_width", title_style_cb, title, TRUE);
	kiba_prefs_add_callback (generalFile, "title_border_style", title_style_cb, title, TRUE);
}
