/*
 *  Copyright (C) 2007 Daniel Bessler  <danielb@settings-dock.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Daniel Bessler <danielb@settings-dock.org>
 */

#ifndef __PREFS_DBUS_
#define __PREFS_DBUS_

#include <gtk/gtk.h>

G_BEGIN_DECLS

#define PREFS_TYPE_DBUS           (prefs_dbus_get_type ())
#define PREFS_DBUS(obj)           (G_TYPE_CHECK_INSTANCE_CAST ((obj), PREFS_TYPE_DBUS, PrefsDbus))
#define PREFS_DBUS_CLASS(obj)     (G_TYPE_CHECK_CLASS_CAST ((obj), PREFS_DBUS,  PrefsDbusClass))
#define PREFS_IS_DBUS(obj)        (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PREFS_TYPE_DBUS))
#define PREFS_IS_DBUS_CLASS(obj)  (G_TYPE_CHECK_CLASS_TYPE ((obj), PREFS_TYPE_DBUS))
#define PREFS_DBUS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), PREFS_TYPE_DBUS, PrefsDbusClass))

typedef struct _PrefsDbus       PrefsDbus;
typedef struct _PrefsDbusClass  PrefsDbusClass;

struct _PrefsDbus {
	GObject       parent;
};

struct _PrefsDbusClass {
	GObjectClass  parent_class;
	
	void (* int_option_changed) (PrefsDbus   *dbus,
				     const gchar *config_path,
				     const gchar *option_id,
				     gint         value);
	void (* double_option_changed) (PrefsDbus   *dbus,
				     const gchar *config_path,
				        const gchar *option_id,
				        gdouble      value);
	void (* bool_option_changed) (PrefsDbus   *dbus,
				     const gchar *config_path,
				      const gchar *option_id,
				      gboolean     value);
	void (* char_option_changed) (PrefsDbus   *dbus,
				     const gchar *config_path,
				      const gchar *option_id,
				      const gchar *value);
	void (* list_option_changed) (PrefsDbus   *dbus,
				     const gchar *config_path,
				      const gchar *option_id,
				      const gchar *value);
};

PrefsDbus* prefs_dbus_get (void);

void prefs_dbus_fini (void);

void prefs_dbus_set_int (const gchar *config_path,
			 const gchar *option_id,
			 gint         value);

void prefs_dbus_set_float (const gchar *config_path,
			 const gchar *option_id,
			    gfloat      value);

void prefs_dbus_set_bool (const gchar *config_path,
			 const gchar *option_id,
			  gboolean     value);

void prefs_dbus_set_char (const gchar *config_path,
			 const gchar *option_id,
			  const gchar *value);

void prefs_dbus_set_list (const gchar *config_path,
			 const gchar *option_id,
			  const gchar *value);

G_END_DECLS

#endif
